/**
 * Arduino Sketch für SmartHome yourself - Arduino Lehrgang
 * 
 * Tag 15: Zeitmessung und Bateriebetrieb
 * In diesem Sketch stoppen wir die Zeit. Die Messung wird über einen Tastendruck gestartet/gestoppt.
 * Kurz gesagt, wir bauen eine Stoppuhr.
 * 
 * https://www.arduino.cc/reference/de/language/functions/time/millis/
 * 
 * By Daniel Scheidler - Dezember 2019
 */
#include <LiquidCrystal.h>            // Library für das LCD einbinden

// Pins fürs LCD festlegen
const unsigned int LCD_ENABLE_PIN = 8;
const unsigned int LCD_REG_SELECT_PIN = 9;

const unsigned int LCD_DATA4_PIN = 4;
const unsigned int LCD_DATA5_PIN = 5;
const unsigned int LCD_DATA6_PIN = 6;
const unsigned int LCD_DATA7_PIN = 7;

// Pins für Taster im Array festlegen
const unsigned int BUTTON_PINS[2] = {2, 3};


// Anzahl an Millisekunden, die das Programm nach Tastendruck wartet, 
// bevor erneut ein Druck auf einen Taster ausgewertet wird.
const unsigned int BUTTON_DELAY = 300;


// Mehrdimensionales Array für Zeitmessung
// [0][0] = Start 1
// [0][1] = Start 2
// [1][0] = Ende  1
// [1][1] = Ende  2
unsigned long timer[2][2] = {{0, 0}, {0, 0}};
unsigned long buttonPressed[2] = {0, 0};

LiquidCrystal lcd(LCD_ENABLE_PIN, LCD_REG_SELECT_PIN, LCD_DATA4_PIN, LCD_DATA5_PIN, LCD_DATA6_PIN, LCD_DATA7_PIN);  // LCD-Pins festlegen



void setup() {
  lcd.begin(16, 2);                                                               // LCD mit Spalten- und Zeilenanzahl initialisieren
  Serial.begin(38400);                                                            // Serielle Kommunikation initialisieren

  for(int i=0; i < sizeof(BUTTON_PINS); i++){
    pinMode(BUTTON_PINS[i], INPUT);
  }
  delay(500);
}

void loop() {
  handleButtons();                                                                // Taster auf Eingabe prüfen und ggf. behandeln
  refreshLCD(getTimerDauer(0), getTimerDauer(1));                                 // LCD-Anzeige aktualisieren
  delay(50);
}

unsigned long getTimerDauer(int index){
  unsigned long timerStart = timer[0][index];                                     // Startzeit für gewählten Timer ermitteln
  unsigned long timerEnde = timer[1][index]==0 ? millis() : timer[1][index];      // Endzeit für gewählten Timer ermitteln
                                                                                  // wird die aktuelle Zeit als Endpunkt verwendet
                                                                                                                                             
  return timerStart==0 ? 0 : timerEnde - timerStart;                              // Wenn Startzeit gesetzt ist, Differenz von Start- und Endzeit zurückgeben. Sonst 0.
}

void startTimer(int index){
  timer[0][index] = millis();                                                     // Startzeit des Timers setzen
}

void stopTimer(int index){
  timer[1][index] = millis();                                                     // Endzeit des Timers setzen
}

void resetTimer(int index){
  timer[0][index] = 0;                                                            // Startpunkt für Timer zurücksetzen
  timer[1][index] = 0;                                                            // Endpunkt für Timer zurücksetzen
}

void refreshLCD(unsigned long time1, unsigned long time2){
  lcd.setCursor(0,0);
  lcd.print("T1: ");
  printFormattedTime(time1);

  lcd.setCursor(0,1);
  lcd.print("T2: ");
  printFormattedTime(time2);
}

void printFormattedTime(unsigned long timeInMs){
  // Zeit in Millisekunden umrechnen in Stunden/Minuten/Sekunden
  unsigned long sekunde = (timeInMs/1000) % 60;                                   // Das Zeichen: "%" ist der Modulo-Operator
  unsigned long minute  = (timeInMs/60000) % 60;
  unsigned long stunde  = (timeInMs/3600000) % 24;
  Serial.print("Time: ");
  Serial.println(timeInMs);
  Serial.println(sekunde);
  Serial.println(minute);
  Serial.println(stunde);
  char str1[12];
  sprintf(str1, "%02ld:%02ld:%02ld", stunde, minute, sekunde );                    // Zeit in formatiertes Char-Array umwandeln
  lcd.print(str1);
}

void handleButtons(){
  for(int i=0; i < (sizeof(BUTTON_PINS) / sizeof(BUTTON_PINS[0])); i++){          // Für alle Buttons der Reihe nach:
    if(digitalRead(BUTTON_PINS[i]) == HIGH                                        // - Prüfen ob Button gedrückt ist,
       && buttonPressed[i] + BUTTON_DELAY < millis()){                            //   und Wartezeit nach letztem Druck abgelaufen ist.

      buttonPressed[i] = millis();                                                // -> Zeitpunkt des Tastendrucks für Button merken.
      
      if(isTimerRunning(i)){                                                      // -> Wenn Button gedrückt ist und Timer aktiviert, aber nicht gestoppt wurde:
        Serial.print("Stop Timer ");
        stopTimer(i);                                                             //    --> Timer beenden
      } else if(isTimerStopped(i)){                                               // -> Sonst, wenn Timer bereits gestoppt, aber noch nicht zurückgesetzt wurde:
        Serial.print("Reset Timer ");
        resetTimer(i);                                                            //    --> Timer zurücksetzen
      } else {                                                                    // -> Sonst
        Serial.print("Start Timer ");
        startTimer(i);                                                            //    --> Timer starten
      }
      Serial.println(i);
    }
  }
}

boolean isTimerRunning(int index){
  return timer[0][index] > 0 && timer[1][index] == 0;                             // Timer läuft, wenn Startzeit gesetzt, aber die Endzeit noch 0 ist.
}

boolean isTimerStopped(int index){
  return timer[0][index] > 0 && timer[1][index] > 0;                              // Timer wurde gestoppt, wenn Startzeit und Endzeit gesetzt sind.
}
